from pylab import *

rc('text', usetex = True)
rc('font', family = 'serif')
rc('font', size = 12)
rcParams['text.latex.preamble']=[r'\usepackage{amsmath}']



def continuate(phi, delta = 0.3):
    threshold = 2 * pi - delta
    n = len(phi)
    for i in range(n - 1):
        if phi[i + 1] - phi[i] < - threshold:
            phi[i + 1 :] += 2 * pi
        elif phi[i + 1] - phi[i] > threshold:
            phi[i + 1 :] -= 2 * pi
    return phi



R = 5
deg = linspace(0, 360, 361)
theta = radians(deg)
z1 = R * exp(1j * theta)


nf = 16

def f(z, i):
    if i == 0:
        return z
    elif i == 1:
        return z**5
    elif i == 2:
        return 1 / z
    elif i == 3:
        return 1 / z**5
    elif i == 4:
        return (z - 4) / (z - 2)**4
    elif i == 5:
        return z + 40
    elif i == 6:
        return z - 40
    elif i == 7:
        return (z - 3) / (z - 4)
    elif i == 8:
        return z + 6
    elif i == 9:
        return z + 5
    elif i == 10:
        return 1 / (z + 5)
    elif i == 11:
        return 1 / (z**2 + 25)
    elif i == 12:
        return 1 / (z**2 + 16)
    elif i == 13:
        return z / (z**2 + 16)
    elif i == 14:
        return 1 / (z**2 + 36)
    elif i == 15:
        return (z - 1) * (z + 7) * (z - 1j) * (z + 1j) / (z**4 * (z - 8) * (z + 2j) * (z - 2j))
    else:
        return 0

def fname(i):
    if i == 0:
        return 'z'
    elif i == 1:
        return 'z^5'
    elif i == 2:
        return '1 / z'
    elif i == 3:
        return '1 / z^5'
    elif i == 4:
        return '(z - 4) / (z - 2)^4'
    elif i == 5:
        return 'z + 40'
    elif i == 6:
        return 'z - 40'
    elif i == 7:
        return '(z - 3) / (z - 4)'
    elif i == 8:
        return 'z + 6'
    elif i == 9:
        return 'z + 5'
    elif i == 10:
        return '1 / (z + 5)'
    elif i == 11:
        return '1 / (z^2 + 25)'
    elif i == 12:
        return '1 / (z^2 + 16)'
    elif i == 13:
        return 'z / (z^2 + 16)'
    elif i == 14:
        return '1 / (z^2 + 36)'
    elif i == 15:
        return '(z - 1) \, (z + 7) \, (z - 1j) \, (z + 1j) / (z^4 \, (z - 8) \, (z + 2j) \, (z - 2j))'
    else:
        return 0
    
    
    
for i in range(nf):

    z2 = array(list(map(f, z1, i * ones(len(z1)))))

    r = abs(z2)
    rho = log(r)
    phi = degrees(continuate(angle(z2)))

    zmp = int(round((phi[-1] - phi[0]) / 360))

    print()
    print('Z - P = {:d}'.format(zmp))
    print()



    close('all')


    figure(1)

    subplot(2, 1, 1)
    plot(deg, rho)
    ylabel(r'$\rho$')
    xlim(0, 360)
    xticks(linspace(0, 360, int(360 / 30 + 1)))
    grid(True)
    title('$f(z) = ' + fname(i) + '$, $ Z - P = {:d}$'.format(zmp))

    subplot(2, 1, 2)
    plot(deg, phi / 360)
    ylabel(r'$\varphi / 360^\circ$')
    xlabel(r'$\theta \; [^\circ]$')
    xlim(0, 360)
    xticks(linspace(0, 360, int(360 / 30 + 1)))
    grid(True)

    savefig('figs/f{:02d}-ap.png'.format(i), bbox_inches = 'tight')


    figure(2, figsize = (6, 6))

    title('$f(z) = ' + fname(i) + '$, $ Z - P = {:d}$'.format(zmp))

    mr = max(r)
    plot(r * cos(radians(phi)), r * sin(radians(phi)))
    plot(0, 0, 'r+')
    xlim(-mr, mr)
    ylim(-mr, mr)
    xlabel(r'$\Re(z_2)$')
    ylabel(r'$\Im(z_2)$')
    grid(True)

    savefig('figs/f{:02d}-polar.png'.format(i), bbox_inches = 'tight')

